@@ -1,16 +1,20 @@
 package Elive::Entity;
-
-use Moose;
-
 use warnings; use strict;
+use Mouse;
+use Mouse::Util::TypeConstraints;
 
+use Elive::Struct;
 use base qw{Elive::Struct};
+
 use Elive::Util;
 
 use YAML;
 use Scalar::Util qw{weaken};
 use UNIVERSAL;
 
+use Elive::Array;
+__PACKAGE__->has_metadata('_deleted');
+
 =head1 NAME
 
     Elive::Entity - Abstract class for Elive Entities
@@ -21,56 +25,158 @@ This is an abstract class that is inherited by all Elive Entity instances.
 
 It provides a simple mapping from the objects to database entities.
 
+=head2 Stringification
+
+Database entities evalute to their primary key, when used in a string
+context.
+
+    if ($user_obj eq "11223344") {
+        ....
+    }
+
+Arrays of sub-items evaluated, in a string context, to a semi-colon seperated
+string of the individual values sorted.
+
+    my $group = Elive::Entity::Group->retrieve([98765]);
+    if ($group->members eq "11223344;2222222") {
+         ....
+    }
+
+In particular meeting participants stringify to userId=role, eg
+
+    my $participant_list = Elive::Entity::ParticipantList->retrieve([98765]);
+    if ($participant_list->participants eq "11223344;2222222") {
+         ....
+    }
+
 =cut
 
-our %Elive_Objects;
+use overload
+    '""' =>
+    sub {shift->stringify}, fallback => 1;
+
+our %Stored_Objects;
 
 #
-# create inside-out accessors for meta properties. keep our
-# object struct clean so that it only contains database information
+# create metadata properties. NB this will be stored inside out to
+# ensure our object is an exact image of the data.
 #
-our %Meta_Data;
 
-foreach my $accessor (qw/_connection _db_data _deleted/) {
-
-    no strict 'refs';
-
-    *{__PACKAGE__.'::'.$accessor} = sub {
-	my $self = shift;
-	my $ref = $self->_refaddr;
-
-	$Meta_Data{ $ref }{ $accessor } = $_[0]
-	    if @_;
-
-	return $Meta_Data{ $ref }{ $accessor };
-    }
+foreach my $accessor (qw/repository _db_data/) {
+    __PACKAGE__->has_metadata($accessor);
 }
 
 sub _url {
-        my $class = shift;
-	my $connection = shift || $class->_last_connection;
-	my $path = shift;
+    my $class = shift;
+    my $repository = shift || $class->repository;
+    my $path = shift;
 
-        return ($connection->url
-		. '/'
-		. $class->entity_name
-		.'/'.
-		$path);
+    return ($repository->url
+	    . '/'
+	    . $class->entity_name
+	    .'/'.
+	    $path);
 }
 
+=head1 METHODS
+
+=cut
+
 =head2 url
 
     my $url = $user->url
 
 Return a restful url for an object instance. This will include both
-the url of the connection scring and the entity class name. It is used
-internally to uniquely identify and cache objects across connections.
+the url of the repository scring and the entity class name. It is used
+internally to uniquely identify and cache objects across repositorys.
 
 =cut
 
 sub url {
     my $self = shift;
-    return $self->_url($self->connection, $self->stringify_self);
+    return $self->_url($self->repository, $self->stringify);
+}
+
+=head2 construct
+
+    my $user = Entity::User->construct(
+            {userId = 123456,
+             loginName => 'demo_user',
+             role => {
+                roleId => 1
+             });
+
+Construct an entity from data.
+
+=cut
+
+sub construct {
+    my $class = shift;
+    my $data = shift;
+    my %opt = @_;
+
+    die "usage: ${class}->construct( \\%data )"
+	unless (Elive::Util::_reftype($data) eq 'HASH');
+
+    #
+    # This method may also be called to coerce sub-entities into existance.
+    # Don't seem to have any direct way of passing options through!?
+
+    local (%Elive::_construct_opts) = %opt;
+
+    $opt{repository} = delete $opt{connection} || $class->connection;
+
+    my %known_properties;
+    @known_properties{$class->properties} = undef;
+
+    foreach (keys %$data) {
+	warn "unknown property $_"
+	    unless exists $known_properties{$_};
+    }
+    
+    my $self = $class->new($data);
+
+    return $self if ($opt{copy});
+
+    my $repository = $opt{repository};
+    #
+    # Retain one copy of the data for this repository
+    #
+    die "can't construct objects without a repository"
+	unless $repository;
+
+    $self->repository($repository);
+
+    my %primary_key_data = map {$_ => $data->{ $_ }} ($class->primary_key);
+
+    foreach (keys %primary_key_data) {
+	unless (defined $primary_key_data{ $_ }) {
+	    die "can't construct $class without value for primary key field: $_";
+	}
+    }
+
+    my $obj_url = $self->url;
+
+    if (my $cached = $Stored_Objects{ $obj_url }) {
+
+	#
+	# Overwrite the cached object, then reuse it.
+	#
+	die "attempted overwrite of object with unsaved changes ($obj_url)"
+	    if $cached->is_changed;
+
+	%{$cached} = %{$self};
+	$self = $cached;
+    }
+    else {
+	weaken ($Stored_Objects{$obj_url} = $self);
+    }
+
+    my $data_copy = Elive::Util::_clone($self);
+    $data_copy->_db_data(undef);
+    $self->_db_data( $data_copy );
+
+    $self;
 }
 
 sub _freeze {
@@ -98,13 +204,13 @@ sub _freeze {
 
 		if ($is_entity) {
 
-		    if (Elive::Struct::_refaddr($_)) {
+		    if (Scalar::Util::refaddr($_)) {
 
 			$_ = $type->construct(Elive::Util::_clone($_),
 			    copy => 1)
 			    unless (Scalar::Util::blessed($_));
 
-			$_ = $_->_stringify_self;
+			$_ = $_->stringify;
 		    }
 		}
 		elsif ($type =~ m{Bool}i) {
@@ -158,13 +264,12 @@ sub _thaw {
     my @properties = $class->properties;
 
     #
-    # Use this table to fix up a couple of anomolies with the fetched
-    # data versus the documented schema and the operation of the rest
-    # of the system (inserts, updates, querys):
+    # Fix up a couple of anomolies with the fetched data versus the
+    # documented schema and the operation of the rest of the system
+    # (inserts, updates, querys):
     # 1. Entity names returned capitalised: 'LoginName' => 'loginName
     # 2. Primary key returned as Id, rather than <entity_name>Id
     #
-
     my %prop_key_map = map {ucfirst($_) => $_} @properties;
 
     $prop_key_map{Id} = lcfirst($class->entity_name).'Id';
@@ -193,8 +298,8 @@ sub _thaw {
 
 		unless ($val_type eq 'ARRAY') {
 		    #
-		    # A single value will be returned singly. Convert it to
-		    # a single element array
+		    # A single value will be deserialise to a simple
+		    # struct. Convert it to a one element array
 		    #
 		    $val = [$val];
 		    warn "thawing $class coerced element into array for $col"
@@ -255,7 +360,7 @@ sub _thaw {
 		    $_ = 0 if ($_ eq '-0');
 		}
 		else {
-		    die "column $col has unknown type: $type";
+		    die "class $class: column $col has unknown type: $type";
 		}
 	    }
 	} 
@@ -285,7 +390,7 @@ sub _unpack_results {
     }
     elsif ($results_type eq 'HASH') {
 	#
-	# Convert some SOAP/XML constructs to their perl equvalent
+	# Convert some SOAP/XML constructs to their perl equivalents
 	#
 	foreach my $key (keys %$results) {
 	    my $value = $results->{$key};
@@ -300,7 +405,7 @@ sub _unpack_results {
 		    $value = [];
 		}
 		#
-		# Throw away our parse of this struct it only exists to
+		# Throw away our parse of this struct. It only exists to
 		# house this collection
 		#
 		return $class->_unpack_results($value);
@@ -313,8 +418,9 @@ sub _unpack_results {
 
 			$value =  $value->{Entry};
 			#
-			# Key, Value pairs.
-			# We're just interested in the value's anyway
+			# Looks like we've got Key, Value pairs.
+			# Throw array the key and reference the value,
+			# that's all we're interested in.
 			#
 			if (Elive::Util::_reftype ($value) eq 'ARRAY') {
 		    
@@ -346,7 +452,7 @@ sub _unpack_results {
     return $results;
 }
 
-sub _get_results {
+sub _check_for_errors {
     my $class = shift;
     my $som = shift;
 
@@ -358,7 +464,10 @@ sub _get_results {
 	if ($class->debug);
 
     if(!Elive::Util::_reftype($result)) {
-	die "unexpected SOAP response: $result";
+	#
+	# Simple scalar
+	#
+	return;
     }
     
     #
@@ -384,10 +493,17 @@ sub _get_results {
 	my @error = grep {defined} ($code, $reason, @stacktrace);
 	die join(' ', @error) || YAML::Dump($result);
     }
+}
+
+sub _get_results {
+    my $class = shift;
+    my $som = shift;
+
+    $class->_check_for_errors($som);
 
     my $results_list;
 
-    $result = $class->_unpack_results($result);
+    my $result = $class->_unpack_results($som->result);
 
     my $reftype = Elive::Util::_reftype($result);
 
@@ -419,15 +535,18 @@ sub _process_results {
 
     my $class = shift;
     my $soap_results = shift;
+    my %opt = @_;
 
-    #  Dereference Adapters. Construct arrays from Collections
-    #  and Maps.
+    #
+    # Thaw our returned SOAP responses to reconstruct the data
+    # image.
+    #
 
     my @rows;
 
     foreach (@$soap_results) {
 
-	my $row = $class->_thaw( $_ );
+	my $row = $class->_thaw($_, %opt);
 
 	push(@rows, $row);
     }
@@ -440,6 +559,11 @@ sub _readback_check {
     my $updates = shift;
     my $rows = shift;
 
+    #
+    # Create and upate responses generally return a copy of the
+    # record, after applying the updates. This routine may be
+    # run to check that the expected updates have been applied
+    #
     die "Didn't receive a response for ".$class->entity_name
 	unless @$rows;
 
@@ -468,6 +592,96 @@ sub _readback_check {
 	    }
 	}
     }
+
+    return $row;
+}
+
+
+sub _cmp_col {
+
+    #
+    # Compare two values for a property 
+    #
+
+    my $class = shift;
+    my $col = shift;
+    my $_v1 = shift;
+    my $_v2 = shift;
+
+    return undef
+	unless (defined $_v1 && defined $_v2);
+
+    my $cmp;
+
+    my ($type, $is_array, $is_def) = Elive::Util::parse_type($class->property_types->{$col});
+    my @v1 = ($is_array? @$_v1: ($_v1));
+    my @v2 = ($is_array? @$_v2: ($_v2));
+
+    if ($is_def) {
+	#
+	# Normalise objects and references to simple strings
+	#
+	for (@v1, @v2) {
+	    #
+	    # autobless references
+	    if (Scalar::Util::refaddr($_)) {
+
+		$_ = $type->construct(Elive::Util::_clone($_))
+		    unless (Scalar::Util::blessed($_));
+
+		$_ = $_->stringify;
+	    }
+	}
+    }
+
+    @v1 = sort @v1;
+    @v2 = sort @v2;
+
+    #
+    # unequal arrays lengths => unequal
+    #
+
+    $cmp ||= scalar @v1 <=> scalar @v2;
+
+    if ($cmp) {
+    }
+    elsif (scalar @v1 == 0) {
+
+	#
+	# Empty arrays => equal
+	#
+
+	$cmp = undef;
+    }
+    else {
+	#
+	# compare values
+	#
+	for (my $i = 0; $i < @v1; $i++) {
+
+	    my $v1 = $v1[$i];
+	    my $v2 = $v2[$i];
+
+	    if ($is_def || $type =~ m{Str}i) {
+		# string comparision. works on simple strings and
+		# stringified entities.
+		# 
+		$cmp ||= $v1 cmp $v2;
+	    }
+	    elsif ($type =~ m{Bool}i) {
+		# boolean comparison
+		$cmp ||= ($v1? 1: 0) <=> ($v2? 1: 0);
+	    }
+	    elsif ($type =~ m{Int}i) {
+		# int comparision
+		$cmp ||= $v1 <=> $v2
+	    }
+	    else {
+		die "class $class: column $col has unknown type: $type";
+	    }
+	}
+    }
+    return $cmp;
 }
 
 =head2 is_changed
@@ -484,7 +698,7 @@ sub is_changed {
     my $db_data = $self->_db_data;
 
     #
-    # not mapped to the database. either a scratch object or
+    # not mapped to a stored data value. either a scratch object or
     # sub entity.
     #
     return unless ($db_data);
@@ -497,9 +711,6 @@ sub is_changed {
 	    || Elive::Util::_reftype($new) ne Elive::Util::_reftype($old)
 	    || $self->_cmp_col($col,$new,$old)) {
 
-	    warn $self->entity_name."#".$self->id." column $col updated from $old to $new"
-		if $self->debug;
-
 	    push (@updated_properties, $col);
 	}
     }
@@ -507,33 +718,55 @@ sub is_changed {
     return @updated_properties;
 }
 
+=head2 set
+
+    $obj->set(prop1 => val1, prop2 => val2 [,...])
+
+Set entity properties.
+
+=cut
+
+sub set {
+    my $self = shift;
+
+    die "attempted to modify  data in a deleted record"
+	if ($self->_deleted);
+
+    return $self->SUPER::set(@_);
+}
+
 =head2 insert
 
-You can insert newly constructed entities:
-
+   # Construct object, then insert.
+   #
    my $new_user = Elive::Entity::User->construct({userId => 12345, 
                                  loginName => 'demo_user',
                                  role => {roleId => 1},
                                });
    $new_user->insert;
 
-Or you can construct and insert the new entity directly:
-
-    my $new_user = Elive::Entity::User->insert({userId => 12345, 
+   # Construct and insert in one go
+   #
+   my $new_user = Elive::Entity::User->insert({userId => 12345, 
                                  loginName => 'demo_user',
                                  role => {roleId => 1},
                                });
 
-Adds a new record to the database.
+Adds a new record to the Elluminate database.
 
 =cut
 
 sub insert {
     my $class = shift;
 
-    if (ref($class)) {
+    if (my $_class = ref($class)) {
 	my $self  = $class;
-	return $self->_insert_class($self,@_);
+	$class = $_class;
+	#
+	# Undo any prior deletion
+	#
+	$self->_deleted(0);
+	return $class->_insert_class($self,@_);
     }
 
     return $class->_insert_class(@_);
@@ -548,7 +781,7 @@ sub _insert_class {
 	unless (Elive::Util::_reftype($insert_data) eq 'HASH');
 
     my $connection = ($opt{connection}
-		      || $class->_last_connection);
+		      || $class->connection);
 
     my $db_data = $class->_freeze($insert_data, mode => 'insert');
 
@@ -556,9 +789,12 @@ sub _insert_class {
 
     my $adapter = $opt{adapter} || 'create'.$class->entity_name;
 
+    $class->check_adapter($adapter);
+
     my $som = $connection->call($adapter,
-				 %$db_data,
-				 loginPassword => $login_password,
+				%$db_data,
+				%{$opt{param} || {}},
+				loginPassword => $login_password,
 	);
 
     my $results = $class->_get_results(
@@ -569,17 +805,41 @@ sub _insert_class {
     # Check that the return response has our inserts
     #
     my $rows =  $class->_process_results( $results );
-    $class->_readback_check($insert_data, $rows);
+    my $row = $class->_readback_check($insert_data, $rows);
 
-    my $self = $class->construct( $rows->[0], connection => $connection );
+    my $self = $class->construct( $row, repository => $connection );
     return $self;
 }
 
+=head2 live_entity
+
+Entity has already been loaded. Return it for reuse.
+
+=cut
+
+sub live_entity {
+    my $class = shift;
+    my $url = shift;
+
+    return $Stored_Objects{ $url };
+}
+
+=head2 live_entities
+
+Return the list of live entities
+
+=cut
+
+sub live_entities {
+    my $class = shift;
+    return \%Stored_Objects;
+}
+
 =head2 update
 
-    Apply our updates to the server. This call will commit outstanding
-    changes to the object and also apply any further updates passed
-    as parameters.
+Apply our updates to the server. This call will commit outstanding
+changes to the object and also apply any further updates passed
+as parameters.
 
 =head3 examples
 
@@ -638,8 +898,12 @@ sub update {
 
     my $adapter = $opt{adapter} || 'update'.$self->entity_name;
 
+    $self->check_adapter($adapter);
+
     my $som =  $self->connection->call($adapter,
-				       %$db_updates);
+				       %$db_updates,
+				       %{$opt{param} || {}},
+);
 
     my $results = $self->_get_results(
 	$som,
@@ -649,167 +913,32 @@ sub update {
     # Check that the return response has our updates
     #
     my $rows =  $self->_process_results( $results );
-    $self->_readback_check(\%updates, $rows);
-    $self->set( %{ $rows->[0] } );
+    my $row = $self->_readback_check(\%updates, $rows);
+    $self->set($row);
 
     #
     # Save the db image
     #
     my $db_data = $self->construct(Elive::Util::_clone($self),
 	copy => 1);
+    #
+    # Make sure our db data hasn't got db data!
+    #
+    $db_data->_db_data(undef);
     $self->_db_data($db_data);
 
     return $self;
 }
 
-=head2 set
-
-    $obj->set(col1 => val1, col2 => val2 [,...])
-
-Assign values to entity object columns.
-
-=cut
-
-sub set {
-    my $self = shift;
-    my %data = @_;
-
-    die "attempted to modify  data in a deleted record"
-	if ($self->_deleted);
-
-    my %entity_column = map {$_ => 1} ($self->properties);
-    my %primary_key = map {$_ => 1} ($self->primary_key);
-
-    foreach (keys %data) {
-
-	if ($entity_column{$_}) {
-
-	    if (exists $primary_key{ $_ }) {
-
-		my $old_val = $self->{$_};
-
-		if (defined $old_val && !defined $data{$_}) {
-		    die "attempt to delete primary key";
-		}
-		elsif ($self->_cmp_col($old_val, $data{$_})) {
-		    die "attempt to update primary key";
-		}
-	    }
-
-	    $self->{$_} = $data{$_};
-	}
-	else {
-	    die "no such column: $_";
-	}
-    }
-
-    return $self;
-}
-
-=head2 construct
-
-    my $user = Elive::Entity::User->construct(
-            {userId = 123456,
-             loginName => 'demo_user',
-             role => {
-                roleId => 1
-             });
-
-Construct an entity from data.
-
-=cut
-
-sub construct {
-    my $class = shift;
-    my $data = shift;
-    my %opt = @_;
-
-    die "usage: ${class}->construct( \\%data )"
-	unless (Elive::Util::_reftype($data) eq 'HASH');
-
-    my $connection = ($opt{connection} ||= $class->_last_connection);
-
-    $data = $class->SUPER::construct(Elive::Util::_clone($data),
-				     %opt);
-
-    #
-    # just want a lightweight struct - not a full blown entity
-    #
-    return $data if ($opt{copy});
-
-    #
-    # Retain one copy of the data for this connection
-    #
-
-    die "can't construct objects without a connection"
-	unless $connection;
-
-    my %primary_key_data = map {$_ => $data->{ $_ }} ($class->primary_key);
-
-    foreach (keys %primary_key_data) {
-	unless (defined $primary_key_data{ $_ }) {
-	    warn YAML::Dump($data)
-		if($class->debug);
-	    die "can't construct $class without value for primary key field: $_";
-	}
-    }
-    
-    my @pkey = (
-	map {$primary_key_data{ $_ }}
-	($class->primary_key)
-	);
-
-    my $obj_url = $class->_url(
-	$connection,
-	$class->_stringify(@pkey)
-	);
-
-    warn"$class key is $obj_url" if ($class->debug);
-
-    my $self;
-
-    if (my $cached = $Elive_Objects{ $obj_url }) {
-
-	#
-	# Reuse the cached copy
-	#
-	die "attempted overwrite of object with unsaved changes ($obj_url)"
-	    if $cached->is_changed;
-
-	%{$cached} = %{$data};
-	$self = $cached;
-    }
-    else {
-	$self = $data;
-	weaken ($Elive_Objects{$obj_url} = $self);
-    }
-
-    my $data_copy = Elive::Util::_clone($data);
-
-    $class->SUPER::construct($data_copy, copy => 1);
-    $self->_db_data( $data_copy );
-
-    $self->connection( $connection ) if ($connection);		   
-
-    #
-    # Save original data, as read from the database
-    #
-
-    warn YAML::Dump({
-	construct => {$class => $data},
-	self => $self,
-		    })
-	if ($class->debug);
-
-    return $self;
-}
-
 =head2 list
 
-    Elive::SomeClass->list(filter => ...)
+    my $users = Elive::Entity::Users->list(filter => 'surname = smith')
 
 Retrieve a list of objects from a table.
 
+Note: this method is not applicable to Elive::Entity::MeetingParameters
+or Elive::Entity::ParticipantList.
+
 =cut
 
 sub list {
@@ -822,10 +951,8 @@ sub list {
 	push( @params, filter => $filter );
     }
 
-    push (@params, adapter => $class->adapter || 'default');
-
     my $connection = ($opt{connection}
-		      || $class->_last_connection
+		      || $class->connection
 	)
 	or die "no connection active";
 
@@ -834,16 +961,19 @@ sub list {
     die "class $class has neither a collection_name or entity_name"
 	unless $collection_name;
 
-    my $som =  $connection->call('list'.$collection_name, @params);
+    my $adapter = $opt{adapter} || 'list'.$collection_name;
+    $class->check_adapter($adapter);
+
+    my $som =  $connection->call($adapter, @params);
 
     my $results = $class->_get_results(
 	$som,
 	);
 
-    my $rows =  $class->_process_results( $results );
+    my $rows =  $class->_process_results($results, %opt);
 
     return [
-	map { $class->construct( $_, connection => $connection) }
+	map { $class->construct( $_, respository => $connection) }
 	@$rows
 	];
 }
@@ -856,7 +986,7 @@ sub _fetch {
     die "usage: ${class}->_fetch( \\%query )"
 	unless (Elive::Util::_reftype($db_query) eq 'HASH');
 
-    my $connection = $opt{connection} || $class->_last_connection
+    my $connection = $opt{connection} || $class->connection
 	or die "no connection active";
 
     my $adapter = $opt{adapter} || 'get'.$class->entity_name;
@@ -864,6 +994,8 @@ sub _fetch {
     warn "get: entity name for $class: ".$class->entity_name.", adapter: ".$adapter
 	if $class->debug;
 
+    $class->check_adapter($adapter);
+
     my $som =  $connection->call($adapter,
 				 %$db_query);
 
@@ -871,7 +1003,7 @@ sub _fetch {
 	$som,
 	);
 
-    my $rows = $class->_process_results( $results );
+    my $rows = $class->_process_results($results, %opt);
     #
     # 0 results => not found
     #
@@ -886,14 +1018,14 @@ sub _fetch {
     #
     # Got one!!
     #
-    return [map {$class->construct( $_, connection => $connection )} @$rows];
+    return [map {$class->construct( $_, repository => $connection )} @$rows];
 }
 
 =head2 retrieve
 
     my $user = Elive::Entity::User->retrieve($user_id)
 
-    Retrieve a single entity objects by primary key.
+Retrieve a single entity objects by primary key.
 
 =cut
 
@@ -902,7 +1034,7 @@ sub retrieve {
     my $vals = shift;
     my %opt = @_;
 
-    die 'usage $class->retrieve_all([$val,..],%opt)'
+    die 'usage $class->retrieve([$val,..],%opt)'
 	unless Elive::Util::_reftype($vals) eq 'ARRAY';
     
     my @key_cols =  $class->primary_key;
@@ -915,20 +1047,20 @@ sub retrieve {
 	    unless defined ($vals->[$n]);
     }
 
-    my $connection = $opt{connection} || $class->_last_connection
+    my $connection = $opt{connection} || $class->connection
 	or die "no connected";
 
     if ($opt{reuse}) {
 
 	#
-	# Have we  already got the object loaded? if so return it
+	# Have we already got the object loaded? if so return it
 	#
 	my $obj_url = $class->_url(
 	    $connection,
-	    $class->_stringify(@$vals)
+	    $class->stringify(@$vals)
 	    );
 
-	my $cached = $Elive_Objects{$obj_url};
+	my $cached = $class->live_entity($obj_url);
 	return $cached if $cached;
     }
 
@@ -941,7 +1073,6 @@ sub retrieve {
     # We've supplied a full primary key, so can expect 0 or 1 values
     # to be returned.
     #
-
     return $all->[0];
 }
 
@@ -950,7 +1081,7 @@ sub retrieve {
     my $participants
           = Elive::Entity::ParticpiantList->retrieve_all($meeting_id)
 
-    Retrieve entity objects by partial primary key.
+Retrieve entity objects by partial primary key.
 
 =cut
 
@@ -983,10 +1114,10 @@ sub retrieve_all {
 
 =head2 delete
 
-    Delete an entity from the datase.
-
     $user_obj->delete;
 
+Delete an entity from the datase.
+
 =cut
 
 sub delete {
@@ -995,14 +1126,17 @@ sub delete {
     my @primary_key = $self->primary_key;
     my @id = $self->id;
 
-    die "can't delete singleton record"
+    die "entity lacks a primary key - can't delete"
 	unless (@primary_key > 0);
 
     my @params = map {
 	$_ => shift( @id );
     } @primary_key;
 
-    my $som =  $self->connection->call('delete'.$self->entity_name,
+    my $adapter = 'delete'.$self->entity_name;
+    $self->check_adapter($adapter);
+
+    my $som =  $self->connection->call($adapter,
 				       @params);
 
     my $results = $self->_get_results(
@@ -1043,7 +1177,16 @@ sub revert {
 	|| die "object doesn't have db-data!? - can't cope";
 
     if (@props) {
-	$self->{$_} = $db_data->{$_} for (@props);
+
+	for (@props) {
+
+	    if (exists $db_data->{$_}) {
+		$self->{$_} = $db_data->{$_};
+	    }
+	    else {
+		delete $self->{$_};
+	    }
+	}
     }
     else {
 	%{ $self } = %{ $db_data };
@@ -1052,41 +1195,10 @@ sub revert {
     return $self;
 }
 
-=head2 DEMOLISH
-
-Moose object Destructor
-
-=cut
-
-sub DEMOLISH {
-
-    my ($self) = shift;
-
-    warn "demolish ".$self->entity_name."#$self = ".$self->_refaddr
-	if (Elive->debug >= 2);
-
-    if (my @changed = $self->is_changed) {
-	my $class = ref($self);
-	warn("$class $self destroyed without saving changes to "
-	     . join(', ', @changed));
-    }
-
-    delete $Meta_Data{ $self->_refaddr };
-
-    $self->SUPER::DEMOLISH
-	if $self->can('SUPER::DEMOLISH');
-}
-
-END {
-
-    foreach (values %Elive_Objects) {
-	next unless $_;
-	my $db_data = $_->_db_data;
-	$_->DEMOLISH if $_;
-	$db_data->DEMOLISH if $db_data;
-	$_ = undef;
-    }
+sub _not_available {
+    my $self = shift;
 
+    die "this operation is not available for ". $self->entity_name;
 }
 
 #
@@ -1097,11 +1209,50 @@ use Elive::Entity::Meeting;
 use Elive::Entity::MeetingParameters;
 use Elive::Entity::Participant;
 use Elive::Entity::ParticipantList;
+use Elive::Entity::Preload;
 use Elive::Entity::Recording;
 use Elive::Entity::Role;
 use Elive::Entity::ServerDetails;
 use Elive::Entity::User;
 
+#------ Coercion Rules
+
+# passing some global flags through from our parent conastructor:
+# $Elive::_construct_opts       - this is copy don't register it as an abject
+
+#coerce 'Elive::Array' => from 'ArrayRef[Int]'
+#          => via {Elive::Array->new($_)};
+
+
+coerce 'Elive::Entity::Role' => from 'HashRef'
+          => via { Elive::Entity::Role->new($_) };
+
+coerce 'Elive::Entity::User' => from 'HashRef'
+          => via { Elive::Entity::User->construct($_, %Elive::_construct_opts) };
+
+=head2 DEMOLISH
+
+Standard Moose/Mouse object destructor. Destroying an object with unsaved
+changes produces a warning. Call the revert method to suppress this.
+
+=cut
+
+sub DEMOLISH {
+    my ($self) = shift;
+    my $class = ref($self);
+
+    if (my $db_data = $self->_db_data) {
+	if (my @changed = $self->is_changed) {
+	    warn("$class $self destroyed without saving changes to: "
+		 . join(', ', @changed));
+	}
+	#
+	# Destroy this objects data
+	#
+	$self->_db_data(undef);
+    }
+}
+
 =head1 ADVANCED
 
 =head2 Object Reuse
@@ -1135,39 +1286,18 @@ The following are all equivalent, and all ok:
     push (@{ $p_list->{participants}      }, $user);
     push (@{ $p_list->get('participants') }, $user);
 
-=head2 Stringification
-
-Database entities evalute to their primary key, when used in a string
-context.
-
-    if ($user_obj eq "11223344") {
-        ....
-    }
-
-Arrays of sub-items evaluated, in a string context, to a semi-colon seperated
-string of the individual values sorted.
-
-    my $group = Elive::Entity::Group->retrieve([98765]);
-    if ($group->members eq "11223344;2222222") {
-         ....
-    }
-
-In particular meeting participants stringify to userId=role, eg
-
-    my $participant_list = Elive::Entity::ParticipantList->retrieve([98765]);
-    if ($participant_list->participants eq "11223344;2222222") {
-         ....
-    }
-
 =head2 Extending and Subclassing Entities
 
-Entity instance classes are simply Moose objects that use this class
+Entity instance classes are simply Mouse objects that use this class
 (Elive::Entity) as base. It should be quite possible to extend existing
-entity classes, or to create your own Moose class based on Elive::Entity
+entity classes.
+
+=cut
 
 =head1 SEE ALSO
 
- Moose
+ Elive::Struct
+ Mouse
  overload
 
 =cut
