@@ -1,8 +1,10 @@
 package Elive::Entity::ServerDetails;
 use warnings; use strict;
 
+use Mouse;
+
+use Elive::Entity;
 use base qw{ Elive::Entity };
-use Moose;
 use Scalar::Util;
 
 =head1 NAME
@@ -13,7 +15,9 @@ Elive::Entity::ServerDetails - Server Details entity class
 
 __PACKAGE__->entity_name('ServerDetails');
 
-has 'serverDetailsId' => (is => 'rw', isa => 'PkeyStr', required => 1);
+has 'serverDetailsId' => (is => 'rw', isa => 'Str', required => 1);
+__PACKAGE__->primary_key('serverDetailsId');
+
 has 'address' => (is => 'rw', isa => 'Str');
 has 'alive' => (is => 'rw', isa => 'Bool');
 has 'codebase' => (is => 'rw', isa => 'Str');
@@ -26,6 +30,10 @@ has 'seats' => (is => 'rw', isa => 'Int');
 has 'port' => (is => 'rw', isa => 'Int');
 has 'version' => (is => 'rw', isa => 'Str');
 	
+=head1 METHODS
+
+=cut
+
 =head2 list
 
 my ($server) = Elive::Entity::ServerDetails->list();
